from .apps import Inbox
from .models import Message
from django.contrib.auth.models import User
from django.test import TestCase


class MessageSendTestCase(TestCase):
    def setUp(self):
        self.u1 = User.objects.create(username='someuser')
        self.u2 = User.objects.create(username='someotheruser')

    def test_send_message(self):
        init_value = Message.objects.all().count()

        message, status = Inbox.send_message(self.u1, self.u2, "This is a message")

        after_value = Message.objects.all().count()

        self.assertEqual(init_value + 1, after_value)
        self.assertEqual(status, 200)
        self.assertEqual(message.content, "This is a message")


class MessageReadingTestCase(TestCase):
    def setUp(self):
        self.u1 = User.objects.create(username='someuser')
        self.u2 = User.objects.create(username='someotheruser')

    def test_unread_messages(self):
        Inbox.send_message(self.u1, self.u2, "This is a message")

        unread_messages = Inbox.get_unread_messages(self.u1)
        unread_messages2 = Inbox.get_unread_messages(self.u2)

        self.assertEqual(unread_messages.count(), 0)
        self.assertEqual(unread_messages2.count(), 1)

    def test_reading_messages(self):
        Inbox.send_message(self.u2, self.u1, "This is another message")

        unread_messages = Inbox.get_unread_messages(self.u1)
        self.assertEqual(unread_messages.count(), 1)

        message = Inbox.read_message(unread_messages[0].id)
        unread_messages_after = Inbox.get_unread_messages(self.u1)

        self.assertEqual(message, "This is another message")
        self.assertEqual(unread_messages_after.count(), 0)

    def test_reading_formatted(self):
        message, status = Inbox.send_message(self.u2, self.u1, "This is just another message")

        unread_messages = Inbox.get_unread_messages(self.u1)
        self.assertEqual(unread_messages.count(), 1)

        message = Inbox.read_message_formatted(message.id)
        unread_messages_after = Inbox.get_unread_messages(self.u1)

        self.assertEqual(message, self.u2.username + ": This is just another message")
        self.assertEqual(unread_messages_after.count(), 0)


class ConversationTestCase(TestCase):
    def setUp(self):
        self.u1 = User.objects.create(username='User')
        self.u2 = User.objects.create(username='Admin')
        self.u3 = User.objects.create(username='Postman')
        self.u4 = User.objects.create(username='Chef')

        # Sender U1
        Inbox.send_message(self.u1, self.u2, "This is a message to User 2")
        Inbox.send_message(self.u1, self.u3, "This is a message to User 3")
        Inbox.send_message(self.u1, self.u4, "This is a message to User 4")

        # Sender U2
        Inbox.send_message(self.u2, self.u1, "This is a message to User 1")
        Inbox.send_message(self.u2, self.u3, "This is a message to User 3")
        Inbox.send_message(self.u2, self.u4, "This is a message to User 4")

        # Some more message between U1 and U2
        Inbox.send_message(self.u1, self.u2, "Hey, thanks for sending this message back")
        Inbox.send_message(self.u2, self.u1, "No problem")

    def test_all_conversations(self):
        conversation_partners = Inbox.get_conversations(self.u1)

        self.assertEqual(len(conversation_partners), 3)
        self.assertIn(self.u2, conversation_partners)
        self.assertIn(self.u3, conversation_partners)
        self.assertIn(self.u4, conversation_partners)

        self.assertNotIn(self.u1, conversation_partners)

    def test_single_conversation(self):
        unread_messages = Inbox.get_unread_messages(self.u1)

        self.assertEqual(unread_messages.count(), 2)

        conversation = Inbox.get_conversation(self.u1, self.u2)
        unread_messages_after = Inbox.get_unread_messages(self.u1)

        self.assertEqual(conversation.count(), 4)
        self.assertEqual(unread_messages_after.count(), 2)

        conversation_limited = Inbox.get_conversation(self.u1, self.u2, limit=2, reversed=True)
        self.assertEqual(conversation_limited.count(), 2)

        self.assertEqual(conversation[0].content, "This is a message to User 2")
        self.assertEqual(conversation[len(conversation) - 1].content, "No problem")
        self.assertEqual(conversation_limited[0].content, "No problem")
        self.assertEqual(conversation_limited[len(conversation_limited) - 1].content,
                         "Hey, thanks for sending this message back")



from __future__ import unicode_literals

from django.core.exceptions import ValidationError
from django.db import models
from django.utils import timezone

from django.utils.encoding import python_2_unicode_compatible
from django.conf import settings
from django.utils.translation import ugettext_lazy as _


AUTH_USER_MODEL = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')
AUTH_MESSAGING_MODEL = getattr(settings, 'AUTH_MESSAGING_MODEL')


@python_2_unicode_compatible
class Message(models.Model):
    """
    A private directmessage
    """
    content = models.TextField(_('Content'))
    sender = models.ForeignKey(AUTH_MESSAGING_MODEL, related_name='sent_dm', verbose_name=_("Sender"),
                               on_delete=models.CASCADE)

    sender_user = models.ForeignKey(AUTH_USER_MODEL, related_name='sent_user_dm', verbose_name=_("Sender_User"),
                                    on_delete=models.SET_NULL, null=True)

    recipient = models.ForeignKey(AUTH_MESSAGING_MODEL, related_name='received_dm', verbose_name=_("Recipient"),
                                  on_delete=models.CASCADE)

    sent_at = models.DateTimeField(_("sent at"), null=True, blank=True)
    read_at = models.DateTimeField(_("read at"), null=True, blank=True)
    title = models.CharField(max_length=10, default='', blank=True)  # this title contains the quote ID

    @property
    def unread(self):
        """returns whether the message was read or not"""
        if self.read_at is not None:
            return False
        return True

    def __str__(self):
        return self.content

    def save(self, **kwargs):
        if self.sender == self.recipient:
            raise ValidationError("You can't send messages to yourself")

        if not self.id:
            self.sent_at = timezone.now()
        super(Message, self).save(**kwargs)


__version__ = '0.9.12'

try:
    from django import VERSION as DJANGO_VERSION
    if DJANGO_VERSION >= (1, 7):
        default_app_config = 'directmessages.apps.DirectmessagesConfig'
    else:
        from directmessages.apps import populateInbox
        populateInbox()
except ImportError:
    pass



# Generated by Django 2.2.1 on 2019-08-21 02:48

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('api', '0005_auto_20190820_1556'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Message',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('content', models.TextField(verbose_name='Content')),
                ('sent_at', models.DateTimeField(blank=True, null=True, verbose_name='sent at')),
                ('read_at', models.DateTimeField(blank=True, null=True, verbose_name='read at')),
                ('title', models.CharField(blank=True, default='', max_length=10)),
                ('recipient', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='received_dm', to='api.Company', verbose_name='Recipient')),
                ('sender', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='sent_dm', to='api.Company', verbose_name='Sender')),
                ('sender_user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='sent_user_dm', to=settings.AUTH_USER_MODEL, verbose_name='Sender_User')),
            ],
        ),
    ]



from __future__ import unicode_literals
from django import VERSION as DJANGO_VERSION

Inbox = None

if DJANGO_VERSION >= (1, 7):
    from django.apps import AppConfig

    class DirectmessagesConfig(AppConfig):
        name = 'directmessages'
        label = 'directmessaging'

        def ready(self):
            # For convenience
            from directmessages.services import MessagingService
            global Inbox
            Inbox = MessagingService()

else:

    def populateInbox():

        from directmessages.services import MessagingService
        global Inbox
        Inbox = MessagingService()



from .models import Message
from django.contrib import admin


class MessageAdmin(admin.ModelAdmin):
    model = Message
    list_display = ('id', 'sender', 'content', )


admin.site.register(Message, MessageAdmin)


from __future__ import unicode_literals

from .models import Message
from .signals import message_read, message_sent
from django.core.exceptions import ValidationError
from django.utils import timezone
from django.db.models import Q


class MessagingService(object):
    """
    A object to manage all messages and conversations
    """

    # Message creation

    def send_message(self, sender, sender_user, recipient, message, quote_id):
        """
        Send a new message
        :param sender: company
        :param sender_user: user
        :param recipient: company
        :param message: String
        :return: Message and status code
        """

        if sender == recipient:
            raise ValidationError("You can't send messages to yourself.")

        message = Message(sender=sender, sender_user=sender_user, recipient=recipient, content=str(message),
                          title=quote_id)
        message.save()

        message_sent.send(sender=message, from_user=message.sender, to=message.recipient)

        # The second value acts as a status value
        return message, 200

    # Message reading
    def get_unread_messages(self, user):
        """
        List of unread messages for a specific user
        :param user: user
        :return: messages
        """
        return Message.objects.all().filter(recipient=user, read_at=None)

    def read_message(self, message_id):
        """
        Read specific message
        :param message_id: Integer
        :return: Message Text
        """
        try:
            message = Message.objects.get(id=message_id)
            self.mark_as_read(message)
            return message.content
        except Message.DoesNotExist:
            return ""

    def read_message_formatted(self, message_id):
        """
        Read a message in the format <User>: <Message>
        :param message_id: Id
        :return: Formatted Message Text
        """
        try:
            message = Message.objects.get(id=message_id)
            self.mark_as_read(message)
            return message.sender.username + ": "+message.content
        except Message.DoesNotExist:
            return ""

    # Conversation management

    def get_conversations(self, user):
        """
        Lists all conversation-partners for a specific user
        :param user: User
        :return: Conversation list
        """
        all_conversations = Message.objects.all().filter(Q(sender=user) | Q(recipient=user))

        contacts = []
        for conversation in all_conversations:
            if conversation.sender != user:
                contacts.append(conversation.sender)
            elif conversation.recipient != user:
                contacts.append(conversation.recipient)

        # To abolish duplicates
        return list(set(contacts))

    def get_conversation(self, company1, company2, quote_id, limit=None, reversed=False, mark_read=False):
        """
        List of messages between two users
        :param company1: User
        :param company2: User
        :param quote_id: Title
        :param limit: int
        :param reversed: Boolean - Makes the newest message be at index 0
        :return: messages
        """
        companys = [company1, company2]

        # Newest message first if it's reversed (index 0)
        if reversed:
            order = '-pk'
        else:
            order = 'pk'

        conversation = Message.objects.all().filter(
            sender__in=companys,
            recipient__in=companys,
            title=quote_id).order_by(order)

        if limit:
            # Limit number of messages to the x newest
            conversation = conversation[:limit]

        if mark_read:
            for message in conversation:
                # Just to be sure, everything is read
                self.mark_as_read(message)

        return conversation

    # Helper methods
    def mark_as_read(self, message):
        """
        Marks a message as read, if it hasn't been read before
        :param message: Message
        """

        if message.read_at is None:
            message.read_at = timezone.now()
            message_read.send(sender=message, from_user=message.sender, to=message.recipient)
            message.save()




from setuptools import setup, find_packages
import os
import directmessages

REQUIREMENTS = [
    'Django>=1.5',
]

CLASSIFIERS = [
    'Environment :: Web Environment',
    'Framework :: Django',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: BSD License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 3',
    'Topic :: Software Development :: Libraries :: Application Frameworks',
    'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
]

setup(
    author="Dominic Monn",
    author_email="yacine@grad4.com",
    name='django-full-directmessages',
    version=directmessages.__version__,
    description='Django-Directmessages is a low-level and easy-to-use Django App to manage simple directmessages.',
    long_description=open(os.path.join(os.path.dirname(__file__), 'README.rst')).read(),
    url='',
    license='BSD License',
    platforms=['OS Independent'],
    classifiers=CLASSIFIERS,
    install_requires=REQUIREMENTS,
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
)
